@@ -7,7 +7,7 @@
 # obtaining a copy of this software and associated documentation files
 # (the "Software"), to deal in the Software without restriction,
 # including without limitation the rights to use, copy, modify, merge,
-# publish, distribute, sublicense, and/or sell copies of the Software,
+# publish, distribute, sub-license, and/or sell copies of the Software,
 # and to permit persons to whom the Software is furnished to do so,
 # subject to the following conditions:
 #
@@ -34,19 +34,27 @@ man 7 units (from the Linux Documentation Project 'man-pages' package)
 """
 
 
+import contextlib
+import fnmatch
 import math
 import numbers
+import os
+import os.path
 import sys
 
-__all__ = ['Bit', 'Byte', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB', 'kB', 'MB', 'GB', 'TB', 'PB', 'EB', 'Kib', 'Mib', 'Gib', 'Tib', 'Pib', 'Eib', 'kb', 'Mb', 'Gb', 'Tb', 'Pb', 'Eb']
+__all__ = [
+    'Bit', 'Byte', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB', 'kB', 'MB', 'GB', 'TB', 'PB', 'EB',
+    'Kib', 'Mib', 'Gib', 'Tib', 'Pib', 'Eib', 'kb', 'Mb', 'Gb', 'Tb', 'Pb', 'Eb',
+    'getsize', 'listdir', 'format', 'format_string', 'format_plural'
+]
 
 # Python 3.x compat
 if sys.version > '3':
     long = int
 
-# Constants for refering to prefix systems
-NIST = int(0)
-SI = int(1)
+# Constants for referring to prefix systems
+NIST = int(2)
+SI = int(10)
 
 SI_PREFIXES = ['k', 'M', 'G', 'T', 'P', 'E']
 SI_STEPS = {
@@ -72,6 +80,24 @@ NIST_STEPS = {
     'Ei': 1152921504606846976
 }
 
+######################################################################
+# Set up our module variables/constants
+
+###################################
+# Internal:
+
+# Console repr(), ex: MiB(13.37), or kB(42.0)
+_FORMAT_REPR = '{unit_singular}({value})'
+
+###################################
+# Exposed:
+
+# String representation, ex:13.37 MiB, or 42.0 kB
+format_string = "{value} {unit}"
+
+# Pluralization behavior
+format_plural = False
+
 
 ######################################################################
 # First, the bytes...
@@ -84,7 +110,32 @@ Byte based types fundamentally operate on self._bit_value"""
 
     def __init__(self, value=0, bytes=None, bits=None):
         """Instantiate with `value` by the unit, in plain bytes, or
-bits. Don't supply more than one keyword."""
+bits. Don't supply more than one keyword.
+
+default behavior: initialize with value of 0
+only setting value: assert bytes is None and bits is None
+only setting bytes: assert value == 0 and bits is None
+only setting bits: assert value == 0 and bytes is None
+        """
+        _raise = False
+        if (value == 0) and (bytes is None) and (bits is None):
+            pass
+        # Setting by bytes
+        elif bytes is not None:
+            if (value == 0) and (bits is None):
+                pass
+            else:
+                _raise = True
+        # setting by bits
+        elif bits is not None:
+            if (value == 0) and (bytes is None):
+                pass
+            else:
+                _raise = True
+
+        if _raise:
+            raise ValueError("Only one parameter of: value, bytes, or bits is allowed")
+
         self._do_setup()
         if bytes:
             # We were provided with the fundamental base unit, no need
@@ -115,7 +166,7 @@ converted *to* this unit"""
         return value / float(self._unit_value)
 
     def _setup(self):
-        return (2, 0, 'Byte')
+        return (2, 0, 'Byte', 'Bytes')
 
     def _do_setup(self):
         """Setup basic parameters for this class"""
@@ -125,7 +176,7 @@ power=10 represents 2^10, which is the NIST Binary Prefix for 1 Kibibyte.
 
 Likewise, for the SI prefix classes `base` will be 10, and the `power`
 for the Kilobyte is 3."""
-        (self._base, self._power, self._name) = self._setup()
+        (self._base, self._power, self._name_singular, self._name_plural) = self._setup()
         self._unit_value = self._base ** self._power
 
     def _norm(self, value):
@@ -139,6 +190,23 @@ type"""
         """Return the mathematical base of an instance"""
         return self._base
 
+    @property
+    def binary(self):
+        """Returns the binary representation of an instance in binary 1s and
+0s. Note that for very large numbers this will mean a lot of 1s and
+0s. For example, GiB(100) would be represented as:
+
+0b1100100000000000000000000000000000000000
+
+That leading '0b' is normal. That's how Python represents binary."""
+        return bin(int(self.bits))
+
+    @property
+    def bin(self):
+        """Alias for instance.binary. Returns the binary representation of an
+instance in binary 1s and 0s."""
+        return self.binary
+
     @property
     def bits(self):
         """Return the number of bits in an instance"""
@@ -149,14 +217,6 @@ type"""
         """Return the number of bytes in an instance"""
         return self._byte_value
 
-    @property
-    def unit(self):
-        """Return the string that is this instances prefix unit name
-
-For instance, KiB(1).unit == 'KiB', Byte(1024).unit == 'Byte', and
-Gb(1).unit == 'Gb'"""
-        return self._name
-
     @property
     def power(self):
         """Return the mathematical power of an instance"""
@@ -176,28 +236,51 @@ Gb(1).unit == 'Gb'"""
             raise ValueError("Instances mathematical base is an unsupported value: %s" % (
                 str(self._base)))
 
+    @property
+    def unit(self):
+        """Return the string that is this instances prefix unit name
+in agreement with this instance value (singular or plural). Following
+the convention that only 1 is singular. This method will always return
+the singular form when bitmath.format_plural is False (default value).
+
+For instance, when plural form is enabled, KiB(1).unit == 'KiB',
+Byte(0).unit == 'Bytes', Byte(1).unit == 'Byte', Byte(1.1).unit == 'Bytes'
+and Gb(2).unit == 'Gbs'"""
+        global format_plural
+
+        if self.prefix_value == 1:
+            # If it's a '1', return it singular, no matter what
+            return self._name_singular
+        elif format_plural:
+            # Pluralization requested
+            return self._name_plural
+        else:
+            # Pluralization NOT requested, and the value is not 1
+            return self._name_singular
+
+    @property
+    def unit_plural(self):
+        """Return the string that is this instances prefix unit name in the
+plural form.
+
+For instance, KiB(1).unit_plural == 'KiB', Byte(1024).unit_plural == 'Bytes',
+and Gb(1).unit_plural == 'Gb'"""
+        return self._name_plural
+
+    @property
+    def unit_singular(self):
+        """Return the string that is this instances prefix unit name in the
+singular form.
+
+For instance, KiB(1).unit_singular == 'KiB', Byte(1024).unit == 'B', and
+Gb(1).unit_singular == 'Gb'"""
+        return self._name_singular
+
     @property
     def value(self):
         """Returns the "prefix" value of an instance"""
         return self.prefix_value
 
-    @property
-    def binary(self):
-        """Returns the binary representation of an instance in binary 1s and
-0s. Note that for very large numbers this will mean a lot of 1s and
-0s. For example, GiB(100) would be represented as:
-
-0b1100100000000000000000000000000000000000
-
-That leading '0b' is normal. That's how Python represents binary."""
-        return bin(int(self.bits))
-
-    @property
-    def bin(self):
-        """Alias for instance.binary. Returns the binary representation of an
-instance in binary 1s and 0s."""
-        return self.binary
-
     @classmethod
     def from_other(cls, item):
         """Factory function to return instances of `item` converted in a new
@@ -214,11 +297,15 @@ instantiate the class ahead of time.
 
 *Example:*
 >>> import bitmath
->>> kib = bitmath.KiB.from_other(MiB(1))
+>>> kib = bitmath.KiB.from_other(bitmath.MiB(1))
 >>> print kib
 KiB(1024.0)
 """
-        return cls(bits=item.bits)
+        if isinstance(item, Byte):
+            return cls(bits=item.bits)
+        else:
+            raise ValueError("The provided items must be a valid bitmath class: %s" %
+                             str(item.__class__))
 
     ######################################################################
     # The following implement the Python datamodel customization methods
@@ -227,14 +314,14 @@ KiB(1024.0)
 
     def __repr__(self):
         """Representation of this object as you would expect to see in an
-intrepreter"""
-        return "%s(%s)" % \
-            (self._name, self.prefix_value)
+interpreter"""
+        global _FORMAT_REPR
+        return self.format(_FORMAT_REPR)
 
     def __str__(self):
         """String representation of this object"""
-        return "%s%s" % \
-            (self.prefix_value, self._name)
+        global format_string
+        return self.format(format_string)
 
     def format(self, fmt):
         """Return a representation of this instance formatted with user
@@ -248,7 +335,9 @@ supplied syntax"""
             'power': self.power,
             'system': self.system,
             'unit': self.unit,
-            'value': self.value,
+            'unit_plural': self.unit_plural,
+            'unit_singular': self.unit_singular,
+            'value': self.value
         }
 
         return fmt.format(**_fmt_params)
@@ -563,8 +652,7 @@ context; TypeError will be raised instead."""
             return self._byte_value / float(other.bytes)
 
     def __truediv__(self, other):
-        # TODO: Fix truediv for num / bm
-        # https://github.com/tbielawa/bitmath/issues/2
+        # num / bm
         return self.__div__(other)
 
     # def __floordiv__(self, other):
@@ -611,6 +699,10 @@ RTYPE. E.g., 3 * MiB(3), or 10 / GB(42)
         # num / bm = num
         return other / float(self.value)
 
+    def __rtruediv__(self, other):
+        # num / bm = num
+        return other / float(self.value)
+
     """Called to implement the built-in functions complex(), int(),
 long(), and float(). Should return a value of the appropriate type.
 
@@ -618,7 +710,7 @@ If one of those methods does not support the operation with the
 supplied arguments, it should return NotImplemented.
 
 For bitmath purposes, these methods return the int/long/float
-equivalent of the this instances prefix unix value. That is to say:
+equivalent of the this instances prefix Unix value. That is to say:
 
     - int(KiB(3.336)) would return 3
     - long(KiB(3.336)) would return 3L
@@ -704,64 +796,64 @@ bit of x AND of y is 0, otherwise it's 1."""
 # NIST Prefixes for Byte based types
 class KiB(Byte):
     def _setup(self):
-        return (2, 10, 'KiB')
+        return (2, 10, 'KiB', 'KiBs')
 
 
 class MiB(Byte):
     def _setup(self):
-        return (2, 20, 'MiB')
+        return (2, 20, 'MiB', 'MiBs')
 
 
 class GiB(Byte):
     def _setup(self):
-        return (2, 30, 'GiB')
+        return (2, 30, 'GiB', 'GiBs')
 
 
 class TiB(Byte):
     def _setup(self):
-        return (2, 40, 'TiB')
+        return (2, 40, 'TiB', 'TiBs')
 
 
 class PiB(Byte):
     def _setup(self):
-        return (2, 50, 'PiB')
+        return (2, 50, 'PiB', 'PiBs')
 
 
 class EiB(Byte):
     def _setup(self):
-        return (2, 60, 'EiB')
+        return (2, 60, 'EiB', 'EiBs')
 
 
 ######################################################################
 # SI Prefixes for Byte based types
 class kB(Byte):
     def _setup(self):
-        return (10, 3, 'kB')
+        return (10, 3, 'kB', 'kBs')
 
 
 class MB(Byte):
     def _setup(self):
-        return (10, 6, 'MB')
+        return (10, 6, 'MB', 'MBs')
 
 
 class GB(Byte):
     def _setup(self):
-        return (10, 9, 'GB')
+        return (10, 9, 'GB', 'GBs')
 
 
 class TB(Byte):
     def _setup(self):
-        return (10, 12, 'TB')
+        return (10, 12, 'TB', 'TBs')
 
 
 class PB(Byte):
     def _setup(self):
-        return (10, 15, 'PB')
+        return (10, 15, 'PB', 'PBs')
 
 
 class EB(Byte):
     def _setup(self):
-        return (10, 18, 'EB')
+        return (10, 18, 'EB', 'EBs')
 
 
 ######################################################################
@@ -773,7 +865,7 @@ class Bit(Byte):
         self.prefix_value = self._to_prefix_value(self._bit_value)
 
     def _setup(self):
-        return (2, 0, 'Bit')
+        return (2, 0, 'Bit', 'Bits')
 
     def _norm(self, value):
         """Normalize the input value into the fundamental unit for this prefix
@@ -786,61 +878,157 @@ type"""
 # NIST Prefixes for Bit based types
 class Kib(Bit):
     def _setup(self):
-        return (2, 10, 'Kib')
+        return (2, 10, 'Kib', 'Kibs')
 
 
 class Mib(Bit):
     def _setup(self):
-        return (2, 20, 'Mib')
+        return (2, 20, 'Mib', 'Mibs')
 
 
 class Gib(Bit):
     def _setup(self):
-        return (2, 30, 'Gib')
+        return (2, 30, 'Gib', 'Gibs')
 
 
 class Tib(Bit):
     def _setup(self):
-        return (2, 40, 'Tib')
+        return (2, 40, 'Tib', 'Tibs')
 
 
 class Pib(Bit):
     def _setup(self):
-        return (2, 50, 'Pib')
+        return (2, 50, 'Pib', 'Pibs')
 
 
 class Eib(Bit):
     def _setup(self):
-        return (2, 60, 'Eib')
+        return (2, 60, 'Eib', 'Eibs')
 
 
 ######################################################################
 # SI Prefixes for Bit based types
 class kb(Bit):
     def _setup(self):
-        return (10, 3, 'kb')
+        return (10, 3, 'kb', 'kbs')
 
 
 class Mb(Bit):
     def _setup(self):
-        return (10, 6, 'Mb')
+        return (10, 6, 'Mb', 'Mbs')
 
 
 class Gb(Bit):
     def _setup(self):
-        return (10, 9, 'Gb')
+        return (10, 9, 'Gb', 'Gbs')
 
 
 class Tb(Bit):
     def _setup(self):
-        return (10, 12, 'Tb')
+        return (10, 12, 'Tb', 'Tbs')
 
 
 class Pb(Bit):
     def _setup(self):
-        return (10, 15, 'Pb')
+        return (10, 15, 'Pb', 'Pbs')
 
 
 class Eb(Bit):
     def _setup(self):
-        return (10, 18, 'Eb')
+        return (10, 18, 'Eb', 'Ebs')
+
+
+######################################################################
+# Utility functions
+def getsize(path, bestprefix=True, system=NIST):
+    """Return a bitmath instance in the best human-readable representation
+of the file size at `path`. Optionally, provide a preferred unit
+system by setting `system` to either `bitmath.NIST` (default) or
+`bitmath.SI`.
+
+Optionally, set ``bestprefix`` to ``False`` to get ``bitmath.Byte``
+instances back.
+    """
+    _path = os.path.realpath(path)
+    size_bytes = os.path.getsize(_path)
+    if bestprefix:
+        return Byte(size_bytes).best_prefix(system=system)
+    else:
+        return Byte(size_bytes)
+
+
+def listdir(search_base, followlinks=False, filter='*',
+            relpath=False, bestprefix=False, system=NIST):
+    """This is a generator which recurses the directory tree
+`search_base`, yielding 2-tuples of:
+
+* The absolute/relative path to a discovered file
+* A bitmath instance representing the "apparent size" of the file.
+
+    - `search_base` - The directory to begin walking down.
+    - `followlinks` - Whether or not to follow symbolic links to directories
+    - `filter` - A glob (see :py:mod:`fnmatch`) to filter results with
+      (default: ``*``, everything)
+    - `relpath` - ``True`` to return the relative path from `pwd` or
+      ``False`` (default) to return the fully qualified path
+    - ``bestprefix`` - set to ``False`` to get ``bitmath.Byte``
+      instances back instead.
+    - `system` - Provide a preferred unit system by setting `system`
+      to either ``bitmath.NIST`` (default) or ``bitmath.SI``.
+
+.. note:: This function does NOT return tuples for directory entities.
+
+.. note:: Symlinks to **files** are followed automatically
+
+    """
+    for root, dirs, files in os.walk(search_base, followlinks=followlinks):
+        for name in fnmatch.filter(files, filter):
+            _path = os.path.join(root, name)
+            if relpath:
+                # RELATIVE path
+                _return_path = os.path.relpath(_path, '.')
+            else:
+                # REAL path
+                _return_path = os.path.realpath(_path)
+
+            if followlinks:
+                yield (_return_path, getsize(_path, bestprefix=bestprefix, system=system))
+            else:
+                if os.path.isdir(_path) or os.path.islink(_path):
+                    pass
+                else:
+                    yield (_return_path, getsize(_path, bestprefix=bestprefix, system=system))
+
+
+@contextlib.contextmanager
+def format(fmt_str=None, plural=False, bestprefix=False):
+    """Context manager for printing bitmath instances.
+
+``fmt_str`` - a formatting mini-language compat formatting string. See
+the @properties (above) for a list of available items.
+
+``plural`` - True enables printing instances with 's's if they're
+plural. False (default) prints them as singular (no trailing 's').
+
+``bestprefix`` - True enables printing instances in their best
+human-readable representation. False, the default, prints instances
+using their current prefix unit.
+    """
+    if 'bitmath' not in globals():
+        import bitmath
+
+    if plural:
+        orig_fmt_plural = bitmath.format_plural
+        bitmath.format_plural = True
+
+    if fmt_str:
+        orig_fmt_str = bitmath.format_string
+        bitmath.format_string = fmt_str
+
+    yield
+
+    if plural:
+        bitmath.format_plural = orig_fmt_plural
+
+    if fmt_str:
+        bitmath.format_string = orig_fmt_str
