@@ -30,7 +30,7 @@ Test to verify the bitmath.Type.to_Type() conversions work
 
 from . import TestCase
 import bitmath
-from bitmath import NIST_STEPS, NIST_PREFIXES
+
 
 class TestToTypeConversion(TestCase):
     def setUp(self):
@@ -60,6 +60,7 @@ class TestToTypeConversion(TestCase):
     def test_from_other(self):
         """MiB object from_other object"""
         mib_from_kib = bitmath.MiB.from_other(bitmath.KiB(1))
+        self.assertIs(type(mib_from_kib), bitmath.MiB)
 
     ##################################################################
     # to b's
@@ -230,3 +231,10 @@ class TestToTypeConversion(TestCase):
         """Convert an SI unit into an NIST unit"""
         kib_from_kb = self.kb.to_KiB()
         self.assertIs(type(kib_from_kb), bitmath.KiB)
+
+    ##################################################################
+    # Naughty bad bad test cases
+    def test_from_other_bad_input(self):
+        """from_other raises if "other" isn't a bitmath instance"""
+        with self.assertRaises(ValueError):
+            a = bitmath.Byte.from_other(str("not a bitmath instance!"))
